/*
  $Id: CrossRefMap.java,v 1.5 2008/12/14 17:05:37 h2b Exp $

  Copyright (C) 2008 Hans-Hermann Bode, Berlin, Germany (h2b.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2,
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.h2b.taglets;

import de.h2b.basis.ClassLabel;
import java.io.*;
import java.util.Hashtable;
import java.util.Map;
import java.util.logging.*;

/**
 * A singleton that provides for a unique map of {@code CrossRefTable}s.
 * <p>
 * The only instance of this object is returned by the {@code getInstance}
 * method. Besides methods {@code getTable} for retrieving tables from the
 * map (which also populate the map with tables as needed), there is a
 * ({@code load}/{@code save} mechanism for the map as a whole which can be
 * used instead of the corresponding mechanism of {@code CrossRefTable}
 * to avoid flooding the directories with auxiliary files.
 * </p>
 *
 * @since h2taglets 0.1
 * @author h2b
 * @version $Revision: 1.5 $, $Date: 2008/12/14 17:05:37 $
 * @see CrossRefTable
 */
@ClassLabel(
    copyright="Copyright (C) 2008 Hans-Hermann Bode",
    ident="$Id: CrossRefMap.java,v 1.5 2008/12/14 17:05:37 h2b Exp $"
)
public final class CrossRefMap {

    private static Logger logger =
    	  Logger.getLogger(CrossRefMap.class.getName());

//Class Methods ---------------------------------------------------------------

    /**
     * Returns the only instance of this singleton.
     *
     * @since h2taglets 0.1
     * @return the instance
     */
    public static CrossRefMap getInstance () {
    	return INSTANCE;
    }

//Public Instance Methods -----------------------------------------------------

   /**
     * Returns the cross-ref table associated to a given key.
     * If the map does not contain the key already, the latter will be
     * added first with a new {@code CrossRefTable} as value.
     *
     * @since h2taglets 0.1
     * @param key the key whose table is in demand
     * @return the {@link CrossRefTable} associated with the key
     */
    public CrossRefTable getTable (String key) {
    	return getTable(key, null);
    }

    /**
     * Returns the cross-ref table associated to a given key.
     * If the map does not contain the key already, the latter will be
     * added first with a new {@code CrossRefTable(path)} as value.
     * If {@code path} is {@code null}, that value is {@code CrossRefTable()}.
     *
     * @since h2taglets 0.1
     * @param key the key whose table is in demand
     * @param path the pathname of the auxiliary file for storing the table
     * @return the {@link CrossRefTable} associated with the key
     * @see CrossRefTable#CrossRefTable(String)
     */
    public synchronized CrossRefTable getTable (String key, String path) {
    	if (!tableMap.containsKey(key)) {
    		tableMap.put(key,
    				path==null? new CrossRefTable(): new CrossRefTable(path));
    		if (logger.isLoggable(Level.FINER)) {
    			logger.finer("Key added to tableMap: \"" + key + "\".");
    			logger.finer("tableMap=" + tableMap);
    		}
    	}
    	return tableMap.get(key);
    }

    /**
     * Loads the map from a file. If the file doesn't exist or
     * an exception occurs during loading, an empty map is created.
     * Note that in both cases any existing entries of the map are destroyed.
     *
     * @since h2taglets 0.1
     * @param f the file the map is to be loaded from
     */
    public synchronized void load (File f) {
    	try {
    		FileInputStream fis = new FileInputStream(f);
    		ObjectInputStream ois = new ObjectInputStream(fis);
    		tableMap = (Map<String,CrossRefTable>)ois.readObject();
    		ois.close();
    	} catch (Exception e) {
    		logger.warning("Could not read file " + f + ".");
    		logger.warning("Creating empty map instead.");
    		tableMap = new Hashtable<String,CrossRefTable>();
		}
    }

    /**
     * Saves the map to a file. If the file is {@code null}, nothing happens.
     * If an exception occurs, a warning message is output.
     *
     * @since h2taglets 0.1
     * @param f the file the map is to be saved to
     */
    public synchronized void save (File f) {
    	if (f==null) return;
    	try {
    		FileOutputStream fos = new FileOutputStream(f);
    		ObjectOutputStream oos = new ObjectOutputStream(fos);
    		oos.writeObject(tableMap);
    		oos.flush();
    		oos.close();
    	} catch (Exception e) {
    		logger.warning(e.toString());
    		logger.warning("+++ Could not write file " + f + ".");
		}
    }

//Private Fields --------------------------------------------------------------

    /**
     * The only instance of this singleton.
     */
    private static final CrossRefMap INSTANCE = new CrossRefMap();

    /**
     * The map of cross-ref tables. The keys are strings, the associated
     * values are {@code CrossRefTable}s.
     */
    private Map<String,CrossRefTable> tableMap =
    	  new Hashtable<String,CrossRefTable>();

//Private Constructors --------------------------------------------------------

    /**
     * Does not allow construction from outside.
     *
     * @since h2taglets 0.1
     */
    private CrossRefMap () {
    	super();
    	logger.fine("CrossRefMap created.");
    }

//End -------------------------------------------------------------------------
}
