/*
  $Id: CrossRefTable.java,v 1.7 2008/12/14 17:05:37 h2b Exp $

  Copyright (C) 2008 Hans-Hermann Bode, Berlin, Germany (h2b.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2,
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.h2b.taglets;

import de.h2b.basis.ClassLabel;
import java.io.*;
import java.util.Vector;
import java.util.logging.*;

/**
 * A helper class that provides for a general cross-referencing mechanism.
 * <p>
 * A cross-referencing table of keys with associated numbers is created.
 * When a key is added to the table, it is ensured that it is not already
 * contained therein, so that the relationship between keys and numbers is
 * unique. Also, numbers are allocated consecutively in accordance  with
 * the adding of keys.
 * </p><p>
 * Two methods are provided to manage the table's elements. Both return
 * numbers as a {@code String} for generality.
 * </p><p>
 * The method {@code getIndex} always returns a number for a given key;
 * if it is not already in the table, it will be added first. Also, this is
 * the only way to add a single element to the table, thus ensuring the
 * desired uniqueness.
 * </p><p>
 * The other method, {@code getReference}, does not change the table.
 * If the key is not found therein, the key itself is returned. This is
 * principally useful when implementing a 2-pass referencing system in
 * conjunction with the {@code save} operation.
 * </p><p>
 * The method {@code save} stores the table to an auxiliary file when one
 * has been specified with the constructor. If the constructor finds such
 * a file, it will read it in to the table. As mentioned, this supports a
 * 2-pass mechanism where forward references are allowed: just use
 * {@code getIndex} in places where the original label to be referenced shall
 * be set and use {@code getReference} anywhere else. In the 1st pass,
 * forward references cannot be resolved but the key name keeps the document
 * readable to some extend; after the 2nd pass, everything will be fine.
 * </p>
 *
 * @since h2taglets 0.1
 * @author h2b
 * @version $Revision: 1.7 $, $Date: 2008/12/14 17:05:37 $
 */
@ClassLabel(
    copyright="Copyright (C) 2008 Hans-Hermann Bode",
    ident="$Id: CrossRefTable.java,v 1.7 2008/12/14 17:05:37 h2b Exp $"
)
public class CrossRefTable implements Serializable {

	private static final long serialVersionUID = -3825276226875003209L;

	private static final Logger logger =
  	  Logger.getLogger(CrossRefTable.class.getName());

//Public Constructors ---------------------------------------------------------

    /**
     * Creates an empty table.
     *
     * @since h2taglets 0.1
     */
    public CrossRefTable () {
    	super();
    	xreftable = new Vector<String>();
    }

    /**
     * Creates a table from an auxiliary file. If the file doesn't exist or
     * an exception occurs during loading, an empty table is created.
     *
     * @since h2taglets 0.1
     * @param path the pathname of the auxiliary file
     */
    public CrossRefTable (String path) {
    	super();
		filepath = path;
    	try {
    		FileInputStream fis = new FileInputStream(path);
    		ObjectInputStream ois = new ObjectInputStream(fis);
    		xreftable = (Vector<String>)ois.readObject();
    		ois.close();
    	} catch (Exception e) {
    		logger.warning(
         	      "Could not read file. Creating empty table instead.");
    		xreftable = new Vector<String>();
		}
    }

//Public Instance Methods -----------------------------------------------------

    /**
     * Returns the index associated to a given key in the cross-ref table.
     * If the table does not contain the key already, the latter will be
     * added first.
     *
     * @since h2taglets 0.1
     * @param key the key whose index is in demand
     * @return the index associated with the key, counting from 1
     */
    public synchronized String getIndex (String key) {
    	int i = 1+xreftable.indexOf(key);
    	if (i<1) { //not found
    		xreftable.addElement(key);
    		i = xreftable.size();
    	}
    	return String.valueOf(i);
    }

    /**
     * Returns the index associated to a given key in the cross-ref table.
     * If the table does not contain the key, the key itself will be
     * returned.
     *
     * @since h2taglets 0.1
     * @param key the key whose index is in demand
     * @return the index associated with the key, counting from 1,
     *         or the key itself if not available
     */
    public String getReference (String key) {
    	int i = 1+xreftable.indexOf(key);
    	if (i<1) return key; //not found
    	return String.valueOf(i);
    }

    /**
     * Saves the table to the auxiliary file. If none has been specified
     * during construction, nothing happens.
     *
     * @since h2taglets 0.1
     */
    public void save () {
    	if (filepath==null) return;
    	try {
    		FileOutputStream fos = new FileOutputStream(filepath);
    		ObjectOutputStream oos = new ObjectOutputStream(fos);
    		oos.writeObject(xreftable);
    		oos.flush();
    		oos.close();
    	} catch (Exception e) {
    		logger.warning("Could not write file.");
		}
    }

//Private Fields --------------------------------------------------------------

    /**
     * The cross-ref table. An element represents a key, it's index the
     * related cross-ref number.
     */
    private Vector<String> xreftable;

    /**
     * The pathname of the auxiliary file for storing the table data.
     * The value {@code null} indicates that there is no such file.
     */
    private String filepath = null;

//End -------------------------------------------------------------------------
}
