/*
  $Id: Registration.java,v 1.5 2008/12/14 17:05:37 h2b Exp $

  Copyright (C) 2008 Hans-Hermann Bode, Berlin, Germany (h2b.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2,
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.h2b.taglets;

import de.h2b.basis.ClassLabel;
import de.h2b.taglets.biblets.*;
import de.h2b.taglets.mathlets.*;
import de.h2b.taglets.reflets.*;
import com.sun.tools.doclets.Taglet;    // Taglet API
import com.sun.tools.doclets.internal.toolkit.taglets.LegacyTaglet;
import java.io.File;
import java.util.Map;                   // Used in register(Map)
import java.util.logging.*;

/**
 * A singleton that provides a static {@code register} method as a single
 * entry point for the javadoc tool.
 * In this way, all taglets of this project will be registered to the map of
 * custom tags at once.
 * <p>
 * Note that this single entry point is not only made available for
 * convenience but is also required for taglets like {@link CiteTaglet}
 * and {@link BiblioTaglet} which have cross-reference dependencies on each
 * other and
 * thus are reliant on shared objects. Apparently, with the current
 * javadoc-tool implementation even singletons are not shared at least
 * by inline and block taglets when registered separately.
 * </p><p>
 * During registration, also the {@link CrossRefMap} is loaded from a file
 * and measure is taken to save the map at shutdown. For this reason,
 * this class implements {@code Runnable}.
 * </p>
 *
 * @see CrossRefMap
 * @since h2taglets 0.1
 * @author h2b
 * @version $Revision: 1.5 $, $Date: 2008/12/14 17:05:37 $
 */
@ClassLabel(
	    copyright="Copyright (C) 2008 Hans-Hermann Bode",
	    ident="$Id: Registration.java,v 1.5 2008/12/14 17:05:37 h2b Exp $"
	)
public final class Registration implements Runnable {

    private static Logger logger =
  	  Logger.getLogger(Registration.class.getName());

//Class Methods ---------------------------------------------------------------

    /**
     * Registers all taglets of this project.
     * Will be called by javadoc tool.
     *
     * @since h2taglets 0.1
     * @param tagletMap  the map to register the taglets to
     */
    public static void register(Map tagletMap) {
    	logger.info("*** registering taglets ***");
    	XREF_MAP.load(XREF_MAP_FILE);
    	register(tagletMap, new MathTaglet());
    	register(tagletMap, new BiblioTaglet());
    	register(tagletMap, new CiteTaglet());
    	register(tagletMap, new LabelTaglet());
    	register(tagletMap, new RefTaglet());
    	if (logger.isLoggable(Level.FINER))
    		logger.finer("tagletMap=" + tagletMap);
    	Runtime.getRuntime().addShutdownHook(new Thread(new Registration()));
    }

    /**
     * Registers a taglet.
     * <p>
     * <em>Implementation note:</em> In the current version of the javadoc
     * tool the map of custom tags consists of key-value pairs with values
     * of an internal type
     * {@code com.sun.tools.doclets.internal.toolkit.taglets.Taglet}.
     * Therefore, taglets of type {@code com.sun.tools.doclets.Taglet}, which
     * are used in this project, have to be converted to the internal
     * interface. The {@code TagletManager} of the internal toolkit will do
     * this automatically only
     * for the last entry added to the map at a single registration, so we
     * have to do the conversion
     * explicitly here. This can be done by creating a {@link LegacyTaglet};
     * beware that the documentation of this class notes:
     * <em>This code is not part of an API.
     * It is implementation that is subject to change.
     * Do not use it as an API.</em>
     * </p>
     *
     * @see LegacyTaglet
     * @see LegacyTaglet#LegacyTaglet(Taglet)
     * @see com.sun.tools.doclets.internal.toolkit.taglets.TagletManager#addCustomTag(String, String)
     * @see com.sun.tools.doclets.internal.toolkit.taglets.TagletManager#checkTaglet
     * @since h2taglets 0.1
     * @param tagletMap  the map to register the taglet to
     * @param taglet the taglet to be registered
     */
    protected static void register(Map tagletMap, Taglet taglet) {
    	String name = taglet.getName();
    	if (logger.isLoggable(Level.FINE))
    		logger.fine("*** registering " + name + " ***");
    	if (tagletMap.get(name) != null) tagletMap.remove(name);
    	tagletMap.put(name, new LegacyTaglet(taglet));
    }

//Public Instance Methods -----------------------------------------------------

    /**
     * Serves as entry point for {@code Runnable} interface.
     *
     * @since h2taglets 0.1
     */
    public void run() {
    	XREF_MAP.save(XREF_MAP_FILE);
    }

//Protected Fields ------------------------------------------------------------

    /**
     * The map that stores the cross-ref tables.
     *
     * @since h2taglets 0.1
     */
    protected static final CrossRefMap XREF_MAP = CrossRefMap.getInstance();

    /**
     * The file that stores the cross-ref map.
     *
     * @since h2taglets 0.1
     */
	protected static final File XREF_MAP_FILE = new File("CrossRefMap.aux");

//Private Constructors --------------------------------------------------------

    /**
     * Does not allow construction from outside.
     *
     * @since h2taglets 0.1
     */
    private Registration () {
    	super();
    	logger.fine("Registration created.");
    }

//End -------------------------------------------------------------------------
}
