/*
  $Id: BiblioTaglet.java,v 1.10 2008/12/14 17:05:37 h2b Exp $

  Copyright (C) 2008 Hans-Hermann Bode, Berlin, Germany (h2b.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2,
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.h2b.taglets.biblets;

import de.h2b.basis.ClassLabel;
import de.h2b.taglets.BlockTaglet;
import de.h2b.taglets.CrossRefMap;
import de.h2b.taglets.CrossRefTable;
import de.h2b.taglets.HtmlUtil;
import com.sun.tools.doclets.Taglet;    // Taglet API
import com.sun.javadoc.*;               // Doclet API
import java.util.Map;                   // Used in register(Map)
import java.util.Arrays;
import java.util.logging.*;

/**
 * A block taglet for typesetting bibliograhical items.
 * The {@code @bibitem} tag is intended to be used in conjunction with the
 * {@code @cite} tag. By providing the same symbolic key as argument
 * to both tags, in the output an identical number enclosed in brackets will
 * appear at both places.
 * For simplicity and to get the references in one pass, a number is allocated
 * to a key at the first occurrence&mdash;whether as citation or as bib item.
 * The scope of this mechanism is limited to the file boundary.
 * <p>
 * The syntax is
 * <b>{{@code @bibitem}</b> <i>key</i><b>,</b> <i>text</i><b>}</b>.
 * Obviously, a key string must not contain a comma;
 * furthermore, to avoid conflicts with future versions you should restrict
 * the key string to alphanumeric characters, periods, colons, hyphens and
 * underlines. Keys are case-sensitive.
 * The text may contain any characters except for a closing brace.
 * </p><p>
 * In the output, the number associated with the <i>key</i>, enclosed in
 * brackets, will be placed in front of the <i>text</i>. Multiple items of
 * one doc comment will be sorted by numbers.
 * </p>
 * For example, the three items in the bibliography block below result from
 * {@code @bibitem math-func, M. Abramowitz...},
 * {@code @bibitem texbook, D. E. Knuth...} and
 * {@code @bibitem latex, L. Lamport...} (trailing text omitted, but items
 * defined in this order). When corresponding {@code @cite} tags appear in a
 * different order <em>before</em> the block tags are declared, like this
 * {@code {@cite texbook,latex,math-func}}
 * {@cite texbook,latex,math-func}, the order is changed as you see below.
 * (Note that the actual {@code @bibitem} tags reside in the block section
 * below.)
 *
 * @bibitem math-func, M. Abramowitz and I. A. Stegun (ed.).
 *                     Handbook of Mathematical Functions. Dover Publications,
 *                     New York (1964). 9th printing, 1970.
 * @bibitem texbook,   D. E. Knuth. The TeXbook.
 *                     Addison-Wesley, Reading, Massachusetts (1986).
 * @bibitem latex,     L. Lamport. LaTeX: A Document Preparation System.
 *                     Addison-Wesley, Reading, Massachusetts (1986).
 * @see CiteTaglet
 * @since h2taglets 0.1
 * @author h2b
 * @version $Revision: 1.10 $, $Date: 2008/12/14 17:05:37 $
 */
@ClassLabel(
    copyright="Copyright (C) 2008 Hans-Hermann Bode",
    ident="$Id: BiblioTaglet.java,v 1.10 2008/12/14 17:05:37 h2b Exp $"
)
public class BiblioTaglet extends BlockTaglet {

    private static Logger logger =
    	  Logger.getLogger(BiblioTaglet.class.getName());

//Public Constructors ---------------------------------------------------------

    /**
     * Creates this taglet.
     *
     * @since h2taglets 0.1
     */
    public BiblioTaglet () {
    	super();
    }

//Class Methods ---------------------------------------------------------------

    /**
     * Registers this taglet.
     *
     * @deprecated Use central registration of
     *             {@link de.h2b.taglets.Registration} instead.
     * @since h2taglets 0.1
     * @param tagletMap  the map to register this tag to
     */
    @Deprecated
    public static void register(Map tagletMap) {
    	logger.warning("+++ BiblioTaglet should not be registered separately!");
    	BiblioTaglet tag = new BiblioTaglet();
    	Taglet t = (Taglet) tagletMap.get(tag.getName());
    	if (t != null) {
    		tagletMap.remove(tag.getName());
    	}
    	tagletMap.put(tag.getName(), tag);
    }

//Public Instance Methods -----------------------------------------------------

    @Override
    public String getName() {
        return NAME;
    }

    /** Returns the processed text of the tag.
     *
     * @param tag the tag to be processed
     * @return the processed text
     * @see de.h2b.taglets.BlockTaglet#toString(com.sun.javadoc.Tag)
     */
    @Override
    public String toString(Tag tag) {
        return OUTPUT_HEADER + tableRow(tag) + OUTPUT_FOOTER;
    }

    /** Returns the processed text of multiple tags.
     * The resulting rows are sorted by numerical bibitem labels.
     *
     * @param tags the tags to be processed
     * @return the processed text
     * @see de.h2b.taglets.BlockTaglet#toString(com.sun.javadoc.Tag)
     */
    @Override
    public String toString(Tag[] tags) {
    	if (tags.length == 0) return null;
    	String[] rows = new String[tags.length];
    	for (int i=0; i<tags.length; i++) rows[i] = tableRow(tags[i]);
    	Arrays.sort(rows);
    	String result = OUTPUT_HEADER;
    	for (String s: rows) result += s;
    	return result + OUTPUT_FOOTER;
    }

//Private Fields --------------------------------------------------------------

    /**
     * The name of this taglet.
     *
     * @since h2taglets 0.1
     */
    private static final String NAME = "bibitem";

    /**
     * The header of this taglet to be output in documentation.
     *
     * @since h2taglets 0.1
     */
    private static final String HEADER = "Bibliography:";

    //Other private constants

    private static final char SEPARATOR_CHAR = ','; //separates key from text

    private static final int MAX_LABEL_WIDTH = 3;
          //expected maximum number of characters of bibitem label
          //excl. brackets

    private static final String OUTPUT_HEADER =
    	  "\n<DT><B>" + HEADER +
    	  "</B></DT><DD><table cellpadding=2 cellspacing=0>";
          //begin of block output

    private static final String OUTPUT_FOOTER = "</table></DD>\n";
          //end of block output

    /**
     * Header for cross-ref table names in this package.
     *
     * @since h2taglets 0.1
     */
    private static String BIBLET_MARKER = "BIBLET::";

    /**
     * The map that stores the cross-ref tables.
     *
     * @since h2taglets 0.1
     */
    private static final CrossRefMap XREF_MAP = CrossRefMap.getInstance();

//Private Methods -------------------------------------------------------------

    /**
     * Returns the tag text formatted for an HTML table row.
     * Leading spaces are added before the bibitem label which will not appear
     * in the visible output but do allow for easy sorting in accordance with
     * {@code MAX_LABEL_WIDTH}.
     *
     * @param tstr the text to be processed
     * @return the processed text
     */
    private String tableRow(Tag tag) {
    	String tstr = tag.text();
    	int j = tstr.indexOf(SEPARATOR_CHAR);
    	String key = j>0? tstr.substring(0, j).trim(): "";
    	String text = tstr.substring(j+1);
    	CrossRefTable crt = XREF_MAP.getTable(BIBLET_MARKER +
    			tag.position().file().getPath());
    	String label = crt.getIndex(key);
    	String result = "<tr><td>";
    	for (int i=0; i<MAX_LABEL_WIDTH-label.length(); i++)
    		result += " "; //add spaces for sorting
    	result += "[" + label + "] " + HtmlUtil.anchor(key, text); //hold sort!
    	return result + "</td></tr>\n";
    }

//End -------------------------------------------------------------------------
}
