/*
  $Id: CiteTaglet.java,v 1.9 2008/12/14 17:05:37 h2b Exp $

  Copyright (C) 2008 Hans-Hermann Bode, Berlin, Germany (h2b.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2,
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.h2b.taglets.biblets;

import de.h2b.basis.ClassLabel;
import de.h2b.taglets.CrossRefMap;
import de.h2b.taglets.CrossRefTable;
import de.h2b.taglets.InlineTaglet;
import de.h2b.taglets.HtmlUtil;
import com.sun.tools.doclets.Taglet;    // Taglet API
import com.sun.javadoc.*;               // Doclet API
import java.util.Map;                   // Used in register(Map)
import java.util.logging.*;

/**
 * An inline taglet for citing bibliographical items.
 * The {@code @cite} tag is intended to be used in conjunction with the
 * {@code @bibitem} tag. By providing the same symbolic key as argument
 * to both tags, in the output an identical number enclosed in brackets will
 * appear at both places; also, a hyperlink is created that points from the
 * citation to the referenced bibitem.
 * For simplicity and to get the references in one pass, a number is allocated
 * to a key at the first occurrence&mdash;whether as citation or as bib item.
 * The scope of this mechanism is limited to the file boundary.
 * <p>
 * The syntax is <b>{{@code @cite}</b> <i>key</i><b>}</b> or
 * <b>{{@code @cite}</b> <i>key1,key2,...</i><b>}</b>, i.e., you can cite
 * several items at once. Obviously, a key string must not contain a comma;
 * furthermore, to avoid conflicts with future versions you should restrict
 * the key string to alphanumeric characters, periods, colons, hyphens and
 * underlines. Keys are case-sensitive.
 * </p>
 * For example, {@code {@cite math-func}} results in the citation
 * {@cite math-func} and {@code {@cite texbook,latex}} yields
 * {@cite texbook,latex}. Note the bibliography block below which arises
 * from corresponding {@code @bibitem} tags.
 *
 * @bibitem math-func, M. Abramowitz and I. A. Stegun (ed.).
 *                     Handbook of Mathematical Functions. Dover Publications,
 *                     New York (1964). 9th printing, 1970.
 * @bibitem texbook,   D. E. Knuth. The TeXbook.
 *                     Addison-Wesley, Reading, Massachusetts (1986).
 * @bibitem latex,     L. Lamport. LaTeX: A Document Preparation System.
 *                     Addison-Wesley, Reading, Massachusetts (1986).
 * @see BiblioTaglet
 * @since h2taglets 0.1
 * @author h2b
 * @version $Revision: 1.9 $, $Date: 2008/12/14 17:05:37 $
 */
@ClassLabel(
    copyright="Copyright (C) 2008 Hans-Hermann Bode",
    ident="$Id: CiteTaglet.java,v 1.9 2008/12/14 17:05:37 h2b Exp $"
)
public class CiteTaglet extends InlineTaglet {

    private static Logger logger =
    	  Logger.getLogger(CiteTaglet.class.getName());

//Public Constructors ---------------------------------------------------------

    /**
     * Creates this taglet.
     *
     * @since h2taglets 0.1
     */
    public CiteTaglet () {
    	super();
    }

//Class Methods ---------------------------------------------------------------

    /**
     * Registers this taglet.
     *
     * @deprecated Use central registration of
     *             {@link de.h2b.taglets.Registration} instead.
     * @since h2taglets 0.1
     * @param tagletMap  the map to register this tag to
     */
    @Deprecated
    public static void register(Map tagletMap) {
    	logger.warning("+++ CiteTaglet should not be registered separately!");
    	CiteTaglet tag = new CiteTaglet();
    	Taglet t = (Taglet) tagletMap.get(tag.getName());
    	if (t != null) {
    		tagletMap.remove(tag.getName());
    	}
    	tagletMap.put(tag.getName(), tag);
    }

//Public Instance Methods -----------------------------------------------------

    @Override
    public String getName() {
        return NAME;
    }

    /** Returns the processed text of the tag.
     *
     * @param tag the tag to be processed
     * @return the processed text
     * @see de.h2b.taglets.InlineTaglet#toString(com.sun.javadoc.Tag)
     */
    @Override
    public String toString(Tag tag) {
    	String s = tag.text(); //String to be parsed
    	String[] keys = s.split(SEPARATOR_REGEX); //separated keys
    	int l = keys.length;
    	CrossRefTable crt = XREF_MAP.getTable(BIBLET_MARKER +
    			tag.position().file().getPath());
    	String result = "[";
    	if (l>0) {
    		String k0 = keys[0].trim();
    		result += HtmlUtil.href(k0, crt.getIndex(k0));
    		for (int i=1; i<l; i++) {
    			String ki = keys[i].trim();
    			result += ", " + HtmlUtil.href(ki, crt.getIndex(ki));
    		}
    	}
    	return result + "]";
    }

//Private Fields --------------------------------------------------------------

    /**
     * The name of this taglet.
     *
     * @since h2taglets 0.1
     */
    private static final String NAME = "cite";

    //Other private constants

    private static final char SEPARATOR_CHAR = ','; //separates multiple keys

    private static final String SEPARATOR_REGEX =
    	  String.valueOf(SEPARATOR_CHAR); //derived regular expression

    /**
     * Header for cross-ref table names in this package.
     *
     * @since h2taglets 0.1
     */
    private static String BIBLET_MARKER = "BIBLET::";

    /**
     * The map that stores the cross-ref tables.
     *
     * @since h2taglets 0.1
     */
    private static final CrossRefMap XREF_MAP = CrossRefMap.getInstance();

//End -------------------------------------------------------------------------
}
