/*
  $Id: MathTaglet.java,v 1.10 2008/12/25 17:01:55 h2b Exp $

  Copyright (C) 2008 Hans-Hermann Bode, Berlin, Germany (h2b.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2,
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.h2b.taglets.mathlets;

import de.h2b.basis.ClassLabel;
import de.h2b.taglets.InlineTaglet;
import com.sun.tools.doclets.Taglet;    // Taglet API
import com.sun.javadoc.*;               // Doclet API
import java.util.Map;                   // Used in register(Map)
import java.io.*;
import java.util.logging.*;

/**
 * An inline taglet for typesetting mathematical formulas.
 * The {@code @math} tag has three variants that allow to include a formula
 * as a graphics or MathML file, as inline MathML or as plain HTML,
 * respectively.
 * <p>
 * The first variant has the general form
 * <b>{{@code @math}</b> <i>attributes</i> <b>file:</b><i>path</i><b>}</b>.
 * The <b>file:</b> indicator is mandatory for this variant and must
 * immediately be followed by a <i>path</i> specification. The taglet will
 * produce an HTML {@code <object/>} element to embed the given file and
 * commit any <i>attributes</i> to it. For example, to include a graphics file
 * you can write {@code {@math style="vertical-align:text-bottom"
 * file:doc-files/example1.gif}} which yields {@math
 * style="vertical-align:text-bottom" file:doc-files/example1.gif}. In the
 * same way you can include an XHTML file containing MathML code by
 * {@code {@math height="45" file:doc-files/example1.xhtml}} resulting in
 * </p><p>
 * {@math height="45" file:doc-files/example1.xhtml}
 * </p><p>
 * (Your browser might not be capable to show that, see below.)
 * </p><p>
 * The second variant has the general form
 * <b>{{@code @math}</b> <i>attributes</i> <b>&lt;mml:</b><i>...</i><b>}</b>.
 * The <i>attributes</i> have the same meaning as above and this time are
 * followed by a
 * valid MathML expression where all MathML elements have to be in the
 * <b>mml</b> namespace. As a matter of principle, both presentation math
 * and content math are possible, but not all browsers can handle MathML.
 * While some can process presentation mode natively, others may need some
 * plugin to work properly; regarding to content mode, I haven't seen any
 * browser that can do it, yet. Anyway, you will notice that for each
 * expression a file will be created in the apidocs directory.
 * </p><p>
 * For instance,
 * <pre>
 * {@code {@math height="45"
  <mml:msup>
    <mml:msqrt>
      <mml:mrow>
        <mml:mi>x</mml:mi>
        <mml:mo>+</mml:mo>
        <mml:mi>y</mml:mi>
      </mml:mrow>
    </mml:msqrt>
    <mml:mn>3</mml:mn>
  </mml:msup>}}
 * </pre>
 * produces the result
 * </p><p>
 * {@math height="45"
  <mml:msup>
    <mml:msqrt>
      <mml:mrow>
        <mml:mi>x</mml:mi>
        <mml:mo>+</mml:mo>
        <mml:mi>y</mml:mi>
      </mml:mrow>
    </mml:msqrt>
    <mml:mn>3</mml:mn>
  </mml:msup>}
 * </p><p>
 * (Again, if your browser does not allow for MathML presentation mode, you
 * will not see the result.)
 * </p><p>
 * The third variant has the general form
 * <b>{{@code @math}</b> <i>formula</i><b>}</b>.
 * In this case, you are using plain HTML to write down the <i>formula</i>,
 * where some substitutions take place for convenience.
 * <ul>
 *   <li>
 *     Spaces are converted to thin spaces.
 *   </li>
 *   <li>
 *     One-letter alphabetic symbols are set in <i>italics</i>; e.g.,
 *     {@code {@math ln x}} results in {@math ln x}.
 *   </li>
 *   <li>
 *     The character immediately following <b>`</b> (left quote) is taken
 *     verbatim. (Provides non-italic typesetting of one-letter symbols.)
 *     For example, {@code {@math `d y/`d x}} results in {@math `d y/`d x}.
 *   </li>
 *   <li>
 *     The alphabetic string immediately following <b>\</b> (backslash) is
 *     converted to an HTML named entity set in italics; e.g.,
 *     {@code {@math \forall \epsilon \gt 0}} results in
 *     {@math \forall \epsilon \gt 0}.
 *     Notice that the italicizing behaviour here principally is intended to
 *     affect Greek letters. If the non-italic variant is needed, the normal
 *     HTML coding can be used (e.g., {@code &pi;} for &pi;).
 *   </li>
 *   <li>
 *     The character sequence enclosed in braces <b>{}</b> or the single
 *     character immediately following <b>_</b> (underscore) is set as
 *     a subscript; e.g.,
 *     {@code {@math a_{i+1}-a_i}} results in {@math a_{i+1}-a_i}.
 *   </li>
 *   <li>
 *     The character sequence enclosed in braces <b>{}</b> or the single
 *     character immediately following <b>^</b> (caret) is set
 *     as a superscript; e.g., {@code {@math a^{i+1}-a^i}} results in
 *     {@math a^{i+1}-a^i}.
 *   </li>
 * </ul>
 * Character sequences used as subscripts or superscripts do recursively
 * underly substitutions. Of course, braces have to be balanced.
 * </p>
 *
 * @since h2taglets 0.1
 * @author h2b
 * @version $Revision: 1.10 $, $Date: 2008/12/25 17:01:55 $
 */
@ClassLabel(
    copyright="Copyright (C) 2008 Hans-Hermann Bode",
    ident="$Id: MathTaglet.java,v 1.10 2008/12/25 17:01:55 h2b Exp $"
)
public class MathTaglet extends InlineTaglet {

    private static Logger logger =
    	  Logger.getLogger(MathTaglet.class.getName());

//Public Constructors ---------------------------------------------------------

    /**
     * Creates this taglet.
     *
     * @since h2taglets 0.1
     */
    public MathTaglet () {
    	super();
    }

//Constants -------------------------------------------------------------------

    /**
     * Indicates file processing.
     *
     *  @since h2taglets 0.1
     */
    public static final String PREFIX_FILE = "file:";

    /**
     * Indicates MathML processing.
     *
     *  @since h2taglets 0.1
     */
    public static final String PREFIX_MATHML = "<mml:";

    /**
     * Name of subdirectory for generated MathML files.
     *
     *  @since h2taglets 0.1
     */
    public static final String SUBDIR_MATHML = "mml-files";

//Class Methods ---------------------------------------------------------------

    /**
     * Registers this taglet.
     *
     * @deprecated Use central registration of
     *             {@link de.h2b.taglets.Registration} instead.
     * @since h2taglets 0.1
     * @param tagletMap  the map to register this tag to
     */
    @Deprecated
    public static void register(Map tagletMap) {
    	logger.warning("+++ MathTaglet should not be registered separately!");
    	MathTaglet tag = new MathTaglet();
    	Taglet t = (Taglet) tagletMap.get(tag.getName());
    	if (t != null) {
    		tagletMap.remove(tag.getName());
    	}
    	tagletMap.put(tag.getName(), tag);
    }

//Public Instance Methods -----------------------------------------------------

    @Override
    public String getName() {
        return NAME;
    }

    /** Returns the processed text of the tag.
     * <p>
     * If {@code tag} contains the {@link #PREFIX_FILE} string, a file-based
     * processing will be done. Otherwise, if {@code tag} contains the
     * {@link #PREFIX_MATHML} string, a MathML-based processing will be done.
     * If none of these conditions apply, a plain HTML-based processing
     * will occur. Prefixes are case-sensitive.
     * </p>
     *
     * @param tag the tag to be processed
     * @return the processed text
     * @see de.h2b.taglets.InlineTaglet#toString(com.sun.javadoc.Tag)
     */
    @Override
    public String toString(Tag tag) {
    	String s = tag.text(); //String to be parsed
    	int i; //index of prefix
    	if ((i=s.indexOf(PREFIX_FILE))>=0)
    		return fileProcessed(s.substring(0, i),
    				             s.substring(i+PREFIX_FILE.length()));
    	else if ((i=s.indexOf(PREFIX_MATHML))>=0)
    		return mathmlProcessed(s.substring(0, i), s.substring(i), tag);
    	else
    		return plainProcessed(s);
    }

//Private Fields --------------------------------------------------------------

    /**
     * The name of this taglet.
     *
     * @since h2taglets 0.1
     */
    private static final String NAME = "math";

    //Escape characters for special processing
    private static final char ESCAPE_SPACE    = ' ';  //denotes space
    private static final char ESCAPE_VERBATIM = '`';  //denotes verbatim processing
    private static final char ESCAPE_NAMED    = '\\'; //denotes named entity
    private static final char ESCAPE_SUB      = '_';  //denotes subscript
    private static final char ESCAPE_SUPER    = '^';  //denotes superscript
    private static final char ESCAPE_LDELIM   = '{';  //denotes left delimiter
    private static final char ESCAPE_RDELIM   = '}';  //denotes right delimiter

//Private Methods -------------------------------------------------------------

    /**
     * Returns tag text processed for file. The filename starts with the
     * beginning of {@code fstr} and ends with the end of the string.
     * <p>
     * From this and the parameter string {@code astr}, we create an HTML
     * {@code <object/>} tag. If the filename is empty, this is not be
     * caught as an error here, but obviously the tag will not work.
     * </p>
     *
     * @since h2taglets 0.1
     * @param astr the arguments to be committed to HTML element. Maybe empty.
     * @param fstr the file path specification. Maybe empty.
     * @return the processed text
     */
    private String fileProcessed(String astr, String fstr) {
    	return "<object data=\"" + fstr + "\" " + astr + ">nil</object>";
    }

    /**
     * Returns tag text processed for MathML. It will create a subdirectory
     * {@code SUBDIR_MATHML} of the output directory, create a unique file
     * therein, write the appropriately prepared MathML content
     * to it and return the filename embedded in an {@code <object/>}
     * element.
     *
     * @since h2taglets 0.1
     * @param astr the arguments to be committed to HTML element. Maybe empty.
     * @param mstr the MathML text to be processed. Starts with MathML prefix.
     * @param tag  the tag to be processed
     * @return the processed text
     */
    private String mathmlProcessed(String astr, String mstr, Tag tag) {
    	String result; //the result of this processing
    	File dir = new File(packPath(tag) + SUBDIR_MATHML); //target directory
    	dir.mkdirs();
    	try {
    		File f = File.createTempFile("math",".xml",dir);
    		FileWriter fw = new FileWriter(f);
    		try {
    			fw.write("<html xmlns=\"http://www.w3.org/1999/xhtml\">\n");
    			fw.write("<head/>\n<body>\n");
    			fw.write("<mml:math " +
    				   "xmlns:mml=\"http://www.w3.org/1998/Math/MathML\">\n");
    			fw.write(mstr);
    			fw.write("\n</mml:math>\n</body>\n</html>\n");
    		} finally {
    			fw.close();
    		}
    		//In the following, do not use f.getAbsolutePath() since site may
    		//be moved to another location (e.g., online):
        	result = "<object data=\"" + SUBDIR_MATHML + "/" + f.getName() +
        	         "\" " + astr + ">nil</object>";
    	} catch (IOException e) {
    		result = "<b><span style=\"color:red\">I/O Exception: " +
    		         e + "--> " + dir.getAbsolutePath() + "</span></b>";
    	}
    	return result;
    }

    /**
     * Returns tag text processed plain.
     * The following essential substitutions will be made:
     * <ul>
     *   <li>Space to {@code &thinsp;}</li>
     *   <li>{@code a} to {@code <i>a</i>}</li>
     *   <li>{@code `c} to {@code c}</li>
     *   <li>{@code \name} to {@code <i>&name;</i>}</li>
     *   <li>{@code _i} to {@code <sub>i</sub>}</li>
     *   <li>{@code _{index}} to {@code <sub>index</sub>}</li>
     *   <li>{@code ^i} to {@code <sup>i</sup>}</li>
     *   <li>{@code ^{index}} to {@code <sup>index</sup>}</li>
     * </ul>
     * where
     * <ul>
     *   <li>{@code a} is any alphabetic character</li>
     *   <li>{@code c} is any character</li>
     *   <li>{@code name} is any alphabetic character sequence</li>
     *   <li>{@code index} is any character sequence</li>
     * </ul>
     *
     * @since h2taglets 0.1
     * @param tstr the text to be processed
     * @return the processed text
     */
    private String plainProcessed(String tstr) {
    	String result = ""; //the result of this processing
    	int l = tstr.length();
    	int i = 0;
    	while (i<l) { //process tstr char by char
    		String s; //to be added to result
    		switch (tstr.charAt(i)) {
    		case ESCAPE_SPACE:
    			s = " ";
    			result += "&thinsp;";
    			break;
    		case ESCAPE_VERBATIM:
    			s = (++i)<l? tstr.substring(i, i+1): "";
    			result += s;
    			break;
    		case ESCAPE_NAMED:
    			s = symbolString(tstr, ++i);
    			result += ("<i>&" + s + ";</i>");
    			      //italics are mainly for Greek letters
    			break;
    		case ESCAPE_SUB:
    			s = indexString(tstr, ++i);
    			result +=
    				  ("<sub><small>" + plainProcessed(s.trim()) +
    				   "</small></sub>");
    			break;
    		case ESCAPE_SUPER:
    			s = indexString(tstr, ++i);
    			result +=
    				  ("<sup><small>" + plainProcessed(s.trim()) +
    				   "</small></sup>");
    			break;
    		default:
    			s = symbolString(tstr, i);
    	        result +=
    	        	  (s.matches("\\A[a-zA-Z]\\Z")? "<i>" + s + "</i>": s);
    	              //italics for one-letter alpha only
    		}
    		i += s.length();
    	}
    	return result;
    }

    /**
     * Returns the next symbol string of the argument beginning with the
     * specified index while characters are in the range [A-Za-z]. If there
     * is no such character, the character at index is returned. If there is
     * no character left at all, an empty string is returned.
     *
     * @since h2taglets 0.1
     * @param str the argument string
     * @param idx the start index
     * @return the symbol string. Minimum 1 character if there is any.
     */
    private String symbolString(String str, int idx) {
    	int l = str.length();
    	int i = idx;
    	if (idx>=l) return ""; //no characters left
    	while (i<l) {
    		char c = str.charAt(i);
    		if (('a'<=c && c<='z') || ('A'<=c && c<='Z')) i++;
    		else break;
    	}
    	return i>idx? str.substring(idx, i): str.substring(idx, idx+1);
    }

    /**
     * Returns the next index string of the argument beginning with the
     * specified index while characters are enclosed in braces. If there
     * are no braces, the character at index is returned. If there is
     * no character left at all, an empty string is returned.
     * <p>
     * Balanced appearance of inner braces is processed suitably.
     * The outermost pair of braces is replaced by spaces. These easily can
     * be trimmed by caller, so that they will not appear in the output. On
     * the other hand, this preserves the caller's counting of already
     * processed characters.
     * </p>
     *
     * @since h2taglets 0.1
     * @param str the argument string
     * @param idx the start index
     * @return the index string. Minimum 1 character if there is any.
     */
    private String indexString(String str, int idx) {
    	int l = str.length();
    	int i = idx;
    	if (idx>=l) return ""; //no characters left
    	char c0 = str.charAt(i++);
    	if (c0!=ESCAPE_LDELIM) return String.valueOf(c0); //single character
    	//We have an opening brace here.
    	int bc = 1; //brace count
    	while (i<l) {
    		char c = str.charAt(i++);
    		if (c==ESCAPE_RDELIM) bc--;
    		else if (c==ESCAPE_LDELIM) bc++;
    		if (bc==0) break;
    	}
    	assert idx+1<=l;
    	if (bc>0) return " " + str.substring(idx+1, l); //unbalanced
    	assert idx+2<=i; //balanced expression contains at least 2 braces
    	return " " + str.substring(idx+1, i-1) + " ";
    	      //replacing braces by spaces
    }

    /**
     * Returns the relative path from the current directory to the output
     * directory of the containing package of the tag. The result will end
     * with a '/' (if not empty) to allow for appending other path elements.
     *
     * @since h2taglets 0.1
     * @param tag  the tag to be processed
     * @return the relative path to the package (suitable for concatenation)
     */
    private String packPath (Tag tag) {
        String p; //qualified package name
        Doc h = tag.holder();
        if (h instanceof ProgramElementDoc)
            p = ((ProgramElementDoc)h).containingPackage().name();
        else if (h instanceof PackageDoc)
            p = h.name();
        else //must be RootDoc
            p = "";
        logger.finest(p);
        if (p.length()==0)
            return "";
        else
            return p.replace('.', '/') + "/";
    }

//End -------------------------------------------------------------------------
}
