/*
  $Id: RefTaglet.java,v 1.5 2008/12/14 17:05:37 h2b Exp $

  Copyright (C) 2008 Hans-Hermann Bode, Berlin, Germany (h2b.de)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2,
  as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package de.h2b.taglets.reflets;

import de.h2b.basis.ClassLabel;
import de.h2b.taglets.CrossRefMap;
import de.h2b.taglets.CrossRefTable;
import de.h2b.taglets.InlineTaglet;
import de.h2b.taglets.HtmlUtil;
import com.sun.tools.doclets.Taglet;    // Taglet API
import com.sun.javadoc.*;               // Doclet API
import java.util.Map;                   // Used in register(Map)
import java.util.logging.*;

/**
 * An inline taglet for numerical referencing.
 * The {@code @ref} tag is intended to be used in conjunction with the
 * {@code @label} tag. By providing the same symbolic key as argument
 * to both tags, an identical number will appear in the output at both places;
 * also, a hyperlink is created that points from the reference to the label.
 * The scope of this mechanism is limited to the file boundary.
 * <p>
 * The number is allocated by the {@code @label} tag. When using the
 * {@code @ref} tag for a forward reference, this number is not known in the
 * first pass of the javadoc tool, instead the symbolic key itself will appear.
 * However, the references will be stored in an auxiliary file; so, in the
 * second run of javadoc, the references will be there. Admittedly,
 * this requires manual deletion of that file if new labels are inserted
 * who disturb the numeric order; this may be done automatically by some
 * <em>clean</em> operation of your development tool.
 * </p><p>
 * The syntax is <b>{{@code @ref}</b> <i>key</i><b>}</b> or
 * <b>{{@code @ref}</b> <i>key1,key2,...</i><b>}</b>, i.e., you can reference
 * several labels at once. Obviously, a key string must not contain a comma;
 * furthermore, to avoid conflicts with future versions you should restrict
 * the key string to alphanumeric characters, periods, colons, hyphens and
 * underlines. Keys are case-sensitive. If you wish to have your reference
 * enclosed by parentheses or something like that, you have to do it yourself.
 * </p><p>
 * For example, {@code {@ref here}} results in the reference {@ref here},
 * when label {@code {@label here}} {@label here} is defined here, and with
 * {@code {@label there}} {@label there} you get {@code {@ref here,there}}
 * {@ref here,there}.
 * </p>
 *
 * @see LabelTaglet
 * @since h2taglets 0.1
 * @author h2b
 * @version $Revision: 1.5 $, $Date: 2008/12/14 17:05:37 $
 */
@ClassLabel(
    copyright="Copyright (C) 2008 Hans-Hermann Bode",
    ident="$Id: RefTaglet.java,v 1.5 2008/12/14 17:05:37 h2b Exp $"
)
public class RefTaglet extends InlineTaglet {

    private static Logger logger =
    	  Logger.getLogger(RefTaglet.class.getName());

//Public Constructors ---------------------------------------------------------

    /**
     * Creates this taglet.
     *
     * @since h2taglets 0.1
     */
    public RefTaglet () {
    	super();
    }

//Class Methods ---------------------------------------------------------------

    /**
     * Registers this taglet.
     *
     * @deprecated Use central registration of
     *             {@link de.h2b.taglets.Registration} instead.
     * @since h2taglets 0.1
     * @param tagletMap the map to register this tag to
     */
    @Deprecated
    public static void register(Map tagletMap) {
    	logger.warning("+++ RefTaglet should not be registered separately!");
    	RefTaglet tag = new RefTaglet();
    	Taglet t = (Taglet) tagletMap.get(tag.getName());
    	if (t != null) {
    		tagletMap.remove(tag.getName());
    	}
    	tagletMap.put(tag.getName(), tag);
    }

//Public Instance Methods -----------------------------------------------------

    @Override
    public String getName() {
        return NAME;
    }

    /** Returns the processed text of the tag.
     *
     * @param tag the tag to be processed
     * @return the processed text
     * @see de.h2b.taglets.InlineTaglet#toString(com.sun.javadoc.Tag)
     */
    @Override
    public String toString(Tag tag) {
    	String s = tag.text(); //String to be parsed
    	String[] keys = s.split(SEPARATOR_REGEX); //separated keys
    	int l = keys.length;
    	CrossRefTable crt = XREF_MAP.getTable(REFLET_MARKER +
    			tag.position().file().getPath());
    	String result = "";
    	if (l>0) {
    		String k0 = keys[0].trim();
    		result += HtmlUtil.href(k0, crt.getReference(k0));
    		for (int i=1; i<l; i++) {
    			String ki = keys[i].trim();
    			result += ", " + HtmlUtil.href(ki, crt.getReference(ki));
    		}
    	}
    	return result;
    }

//Private Fields --------------------------------------------------------------

    /**
     * The name of this taglet.
     *
     * @since h2taglets 0.1
     */
    private static final String NAME = "ref";

    //Other private constants

    private static final char SEPARATOR_CHAR = ','; //separates multiple keys

    private static final String SEPARATOR_REGEX =
    	  String.valueOf(SEPARATOR_CHAR); //derived regular expression

    /**
     * Header for cross-ref table names in this package.
     *
     * @since h2taglets 0.1
     */
    private static String REFLET_MARKER = "REFLET::";

    /**
     * The map that stores the cross-ref tables.
     *
     * @since h2taglets 0.1
     */
    private static final CrossRefMap XREF_MAP = CrossRefMap.getInstance();

//End -------------------------------------------------------------------------
}
